﻿/*
 *
 *
 *    AutoCapSensorV1.cs
 *
 *    Wattmeter interface.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

// define this symbol to enable boxcar averaging
//#define BOXCAR

using System;
using System.Threading;
using System.IO.Ports;
using System.Net;

using KK5JY.IO;

namespace KK5JY.AutoCap {
	/// <summary>
	/// An AutoCap custom SWR meter.
	/// </summary>
	public class AutoCapSensorV1 : ISensor, IStart, IQueryVersion, IDisposable {
		#region Constants
		/// <summary>
		/// The message prefix;
		/// </summary>
		private const char SOH = '#';

		/// <summary>
		/// The message terminator.
		/// </summary>
		private const char ETB = ';';
#if BOXCAR
		/// <summary>
		/// The length of the SWR boxcar average
		/// </summary>
		private const int BoxcarLength = 3; // samples
#endif
		#endregion

		#region Private Fields
		private Thread m_Thread;
		private IDeviceHelper m_Peer;
		private DateTimeOffset m_LastUpdate;
		private DateTimeOffset m_LastPoll;
		private string m_RxBuffer;
		private bool m_AutoPoll;
		private bool m_Quit;
#if BOXCAR
		private decimal[] m_Boxcar;
		private int m_BoxcarIndex;
#endif
		#endregion

		#region Public Events
		/// <summary>
		/// Raised when a new reading is taken.
		/// </summary>
		public event EventHandler<SensorDataEvent> SensorData;
		#endregion

		#region Public Properties
		/// <summary>
		/// Enable automatic polling.
		/// </summary>
		public bool AutoPoll {
			get {
				return m_AutoPoll;
			}
			set {
				if (m_AutoPoll != value) {
					m_AutoPoll = value;
					if (m_AutoPoll) {
						Start();
					} else {
						Stop();
					}
				}
			}
		}

		/// <summary>
		/// Maximum time to wait for a Poll() response before re-polling.
		/// Only applies to auto-polling mode.
		/// </summary>
		private TimeSpan MaximumStale { get; set; }

		/// <summary>
		/// Minimum wait time between polls.  Gives the meter a chance to
		/// reset between polls.
		/// </summary>
		public TimeSpan MinimumPollInterval { get; set; }

		/// <summary>
		/// The version information.
		/// </summary>
		public string VersionString { get; private set; }

		/// <summary>
		/// Set to true to query only the SWR, and not the full data packet.
		/// </summary>
		public bool ReadOnlySWR { get; set; }
		#endregion

		#region Public Methods
        /// <summary>
        /// Common constructor code.
        /// </summary>
        private AutoCapSensorV1() {
            MaximumStale = new TimeSpan(0, 0, 0, 0, 250);
            MinimumPollInterval = new TimeSpan(0, 0, 0, 0, 50);

            m_RxBuffer = "";
#if BOXCAR
			m_Boxcar = new decimal[BoxcarLength];
			m_BoxcarIndex = 0;
#endif
        }

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="portName">The serial port name.</param>
		/// <param name="speed">The speed of the device (bps).</param>
		public AutoCapSensorV1(string portName, int speed = 9600) : this() {
			var port = new SerialPort(portName);
			port.BaudRate = speed;
			port.Parity = Parity.None;
			port.DataBits = 8;
			port.StopBits = StopBits.One;
			m_Peer = new SerialPortHelper(port);
			m_Peer.DataReceived += DataReceived;
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="ep">The remote IP endpoint address.</param>
		public AutoCapSensorV1(IPEndPoint ep) : this() {
			m_Peer = new SocketHelper(ep);
			m_Peer.DataReceived += DataReceived;
		}

		/// <summary>
		/// Manually request a new sample.
		/// </summary>
		public void Poll() {
			try {
				m_LastPoll = DateTimeOffset.UtcNow;
				if (ReadOnlySWR) {
					m_Peer.Write("#SWR;");
				} else {
					m_Peer.Write("#ALL;");
				}
			} catch {
				// nop
			}
		}

		/// <summary>
		/// Start autopolling.
		/// </summary>
		public void Start() {
			if (m_Thread != null) return;
			m_Quit = false;
			m_Peer.Start();
			m_Thread = new Thread(Worker) { Name = "AutoCapSensorV1 AutoPoll Thread" };
			m_Thread.Start();
		}

		/// <summary>
		/// Stop autopolling.
		/// </summary>
		public void Stop() {
			m_Quit = true;
			try { m_Peer.Dispose(); } catch { /* nop */ }
			if (m_Thread == null) return;
			m_Thread.Join();
			m_Thread = null;

		}

		/// <summary>
		/// Stop autopolling.
		/// </summary>
		public void Dispose() {
			Stop();
		}

		/// <summary>
		/// Query the sensor version.
		/// </summary>
		public bool QueryVersion() {
			try {
				m_Peer.Write("#VERSION;");
				return true;
			} catch {
				return false;
			}
		}
		#endregion

		#region Event Handlers and Utilities
		/// <summary>
		/// Raised when new serial data is received.
		/// </summary>
		private void DataReceived(object sender, ReceivedDataEvent e) {
			// read data and update the RX buffer
			m_RxBuffer += e.Data;

			// trim leading stuff
			string current = "";
			bool collecting = false;
			int trimTo = 0;
			for (int i = 0; i != m_RxBuffer.Length; ++i) {
				if (m_RxBuffer[i] == SOH) {
					collecting = true;
					trimTo = i;
				} else if (m_RxBuffer[i] == ETB) {
					collecting = false;
					if (!String.IsNullOrEmpty(current)) {
						ProcessMessage(current);
					}
					current = "";
					trimTo = i + 1;
				} else if (collecting) {
					current += m_RxBuffer[i];
				}
			}
			if (trimTo != 0) {
				m_RxBuffer = m_RxBuffer.Substring(trimTo);
			}

			// if the buffer is now empty, poll again
			if (m_RxBuffer.Length == 0) {
				// wait to avoid hammering the meter
				if (!MinimumPollInterval.Equals(TimeSpan.Zero)) {
					var howLong = DateTimeOffset.UtcNow - m_LastPoll;
					if (howLong < MinimumPollInterval) {
						Thread.Sleep(MinimumPollInterval - howLong);
					}
				}

				// ask for the next reading
				Poll();
			}
		}

		/// <summary>
		/// Parse one complete record.
		/// </summary>
		private void ProcessMessage(string msg) {
			// stamp the update
			m_LastUpdate = DateTimeOffset.UtcNow;

			// trim response
			if (msg.StartsWith("OK:", StringComparison.InvariantCultureIgnoreCase)) {
				msg = msg.Substring(3);
			}

#if VERBOSE_DEBUG
			System.Diagnostics.Debug.WriteLine("DEBUG: AutoCapSensorV1 data: " + msg);
#endif
			// if there is no message data, bail
			if (String.IsNullOrEmpty(msg)) return;

			// parse the message components
			var parts = msg.Split('=');
			if (parts.Length != 2) return;
			var cmd = parts[0];
#if BOXCAR
			var swr = 0m;
#endif
			var evData = new SensorData();
			switch (cmd.ToUpper()) {
				case "SWR":
					evData.Z = null;
					evData.Phi = null;
					evData.ID = "";
					evData.Power = null;
					try {
						evData.SWR = Decimal.Parse(parts[1].Trim());
					} catch {
						return; // this is critical, failure not an option
					}

					// raise a new event
					if (SensorData != null) {
						var newReport = new SensorDataEvent { Data = evData };
						SensorData(this, newReport);
					}
					break;
				case "ALL":
					// re-split the data portion on ','
					parts = parts[1].Split(',');
					if (parts.Length != 3) return;

					evData.Z = null;
					evData.Phi = null;
					evData.ID = "";
					try {
						evData.Power = Decimal.Parse(parts[1]);
#if BOXCAR
						swr = Decimal.Parse(parts[0]);
#else
						evData.SWR = Decimal.Parse(parts[0]);
#endif
#if BOXCAR
						var avg = 0m;
						m_Boxcar[m_BoxcarIndex++] = swr;
						if (m_BoxcarIndex >= BoxcarLength) m_BoxcarIndex = 0;
						for (int i = 0; i != BoxcarLength; ++i) {
							avg += m_Boxcar[i];
						}
						avg /= BoxcarLength;
						m_Data.SWR = avg;
#endif
					} catch {
						return; // these are critical, failure not an option
					}

					// raise a new event
					if (SensorData != null) {
						var newReport = new SensorDataEvent { Data = evData };
						SensorData(this, newReport);
					}
					break;

				case "VERSION":
					VersionString = parts[1];
					break;
			}
		}

		/// <summary>
		/// Worker thread.
		/// </summary>
		private void Worker() {
			// prime the pump
			Poll();

			// then just watchdog the link
			while (!m_Quit) {
				DateTimeOffset now = DateTimeOffset.UtcNow;

				// if the last reading is stale, ask for a new one
				if (now - m_LastUpdate > MaximumStale) {
					m_LastUpdate = now;
					Poll();
				}
				Thread.Sleep(MaximumStale);
			}
		}
		#endregion
	}
}
