﻿/*
 *
 *
 *    Config.cs
 *
 *    AutoCap configuration structures.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Net;
using System.Xml;
using System.Linq;
using System.Collections.Generic;

namespace KK5JY.AutoCap {
	/// <summary>
	/// A tuning threshold.
	/// </summary>
	public class Threshold {
		/// <summary>
		/// Minimum SWR.
		/// </summary>
		public decimal SWR { get; set; }

		/// <summary>
		/// Pulse length / Step count.
		/// </summary>
		public decimal Size { get; set; }

		/// <summary>
		/// The normalized motor effort in [0.0, 1.0]
		/// </summary>
		public decimal Effort { get; set; }

		/// <summary>
		/// Set to true if the 'Size' field is a step count; otherwise, 'Size' is a pulse width, measured in seconds.
		/// </summary>
		public bool IsStep { get; set; }

		/// <summary>
		/// Creates a new threshold limit.
		/// </summary>
		public Threshold() {
			SWR = 2.0m;    // 2:1
			Size = 0.1m;  // 100ms
			Effort = 1.0m; // 100%
			IsStep = false;
		}
	}

	/// <summary>
	/// Motor control types.
	/// </summary>
	public enum MotorTypes {
		/// <summary>
		/// RTS/DTR control.
		/// </summary>
		RS232,

        /// <summary>
        /// H-bridge relays connected to RS-232 RTS/DTR.
        /// </summary>
        RS232Bridge,

		/// <summary>
		/// The first generation AutoCap motor controller.
		/// </summary>
		AutoCapV1,

		/// <summary>
		/// Simulator.
		/// </summary>
		Sim,
	}

	/// <summary>
	/// Sensor types.
	/// </summary>
	public enum SensorTypes {
		/// <summary>
		/// Telepost LP-100A
		/// </summary>
		LP100A,

		/// <summary>
		/// The first generation AutoCap SWR sensor.
		/// </summary>
		AutoCapV1,

		/// <summary>
		/// Simulator.
		/// </summary>
		Sim,
	}

	/// <summary>
	/// Radio interface types.
	/// </summary>
	public enum RadioTypes {
		/// <summary>
		/// Radio feature disabled.
		/// </summary>
		None,

		/// <summary>
		/// Ham Radio Deluxe
		/// </summary>
		HRD,

		/// <summary>
		/// FLDigi
		/// </summary>
		FLDigi,

		/// <summary>
		/// Simulator
		/// </summary>
		Sim,
	}

	/// <summary>
	/// Configuration block for the Core.
	/// </summary>
	public class CoreConfig {
		/// <summary>
		/// The motor control type.
		/// </summary>
		public MotorTypes MotorType { get; set; }

		/// <summary>
		/// The motor control serial port.
		/// </summary>
		public string MotorControlPort { get; set; }

		/// <summary>
		/// The motor polarity is reversed.
		/// </summary>
		public bool MotorPolarityReversed { get; set; }

		/// <summary>
		/// The motor control relays are reversed (for motor controls that have separate POWER and DIRECTION relays).
		/// </summary>
		public bool MotorRelayRolesReversed { get; set; }

		/// <summary>
		/// The output port number, starting at zero.  For motor controllers that support
		/// more than one output port.
		/// </summary>
		public int MotorOutputPort { get; set; }

		/// <summary>
		/// Enables motor braking between movement commands, if supported by the motor.
		/// </summary>
		public bool MotorBrakingEnabled { get; set; }

		/// <summary>
		/// The SWR sensor type.
		/// </summary>
		public SensorTypes SensorType { get; set; }

		/// <summary>
		/// The SWR sensor port.
		/// </summary>
		public string SensorPort { get; set; }
		
		/// <summary>
		/// The amount of 'slop' allowed in SWR readings; when determining that
		/// the SWR is 'higher' than the last reading, the increase must be at
		/// least this much higher than the last reading for a 'true' evaluation.
		/// </summary>
		public decimal SensorEpsilon { get; set; }

		/// <summary>
		/// A collection of thresholds, sorted by SWR.
		/// </summary>
		public Threshold[] Thresholds { get; set; }

		/// <summary>
		/// Read delay in seconds; seconds.
		/// </summary>
		public decimal ReadDelay { get; set; }

		/// <summary>
		/// Step rate for 100% rotation speed.  Used to calculate read delays.
		/// </summary>
		public uint MaxStepRate { get; set; }

		/// <summary>
		/// Indicates that the sensor should only read SWR.
		/// </summary>
		public bool ReadOnlySWR { get; set; }

		/// <summary>
		/// Minimum tuning power in watts.
		/// </summary>
		public decimal MinPower { get; set; }

		/// <summary>
		/// Minimum number of threshold excursions required
		/// to trigger an initial tuning action.
		/// </summary>
		public ushort AlarmCount { get; set; }

		/// <summary>
		/// The radio interface type.
		/// </summary>
		public RadioTypes RadioType { get; set; }

		/// <summary>
		/// The IP address of the radio interface.
		/// </summary>
		public string RadioHost { get; set; }

		/// <summary>
		/// The radio interface port number.
		/// </summary>
		public int RadioPort { get; set; }

		/// <summary>
		/// When true, the algorithm will attempt to update its frequency hint
		/// data after every successful tuning operation.
		/// </summary>
		public bool RadioLearningEnabled { get; set; }

		/// <summary>
		/// Create a new configuration object with default values.
		/// </summary>
		public CoreConfig() {
			this.MinPower = 5.0m; // W
			this.AlarmCount = 3;
			this.MotorControlPort = "None";
			this.MotorPolarityReversed = false;
			this.MotorRelayRolesReversed = false;
			this.MotorType = MotorTypes.RS232;
			this.RadioHost = "localhost";
			this.RadioPort = 8080;
			this.RadioType = RadioTypes.None;
			this.RadioLearningEnabled = true;
			this.ReadDelay = 0.300m; // sec total
			this.MaxStepRate = 100; // sec / step
			this.ReadOnlySWR = false;
			this.SensorPort = "None";
			this.SensorType = SensorTypes.LP100A;
			this.SensorEpsilon = 0.05m; // SWR
			this.Thresholds = new[] {
				new Threshold { SWR = 6.0m, Size = 0.250m, Effort = 1.0m, IsStep = false },
				new Threshold { SWR = 3.0m, Size = 0.200m, Effort = 1.0m, IsStep = false },
				new Threshold { SWR = 2.0m, Size = 0.100m, Effort = 1.0m, IsStep = false },
				new Threshold { SWR = 1.5m, Size = 0.050m, Effort = 1.0m, IsStep = false },
			};
		}

		/// <summary>
		/// Serialize the configuration into an XML structure.
		/// </summary>
		public XmlDocument Serialize() {
			XmlDocument doc = new XmlDocument();
			var root = doc.CreateElement("AutoCapCore");
			// append the assembly version as an attribute to the root
			root.SetAttribute("Version", System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString());
			doc.AppendChild(root);

			// store thresholds
			var ths = doc.CreateElement("Thresholds");
			root.AppendChild(ths);
			foreach (var item in Thresholds) {
				var th = doc.CreateElement("Threshold");
				th.SetAttribute("SWR", item.SWR.ToString());
				th.SetAttribute("Size", item.Size.ToString());
				th.SetAttribute("Effort", item.Effort.ToString());
				th.SetAttribute("IsStep", item.IsStep.ToString());
				ths.AppendChild(th);
			}

			// store motor details
			var motor = doc.CreateElement("Motor");
			motor.SetAttribute("Type", MotorType.ToString());
			motor.SetAttribute("ControlPort", MotorControlPort);
			motor.SetAttribute("OutputPort", MotorOutputPort.ToString());
			motor.SetAttribute("Reverse", MotorPolarityReversed.ToString());
			motor.SetAttribute("ReverseRelay", MotorRelayRolesReversed.ToString());
			motor.SetAttribute("BrakingEnabled", MotorBrakingEnabled.ToString());
			root.AppendChild(motor);

			// store sensor details
			var sensor = doc.CreateElement("Sensor");
			sensor.SetAttribute("Type", SensorType.ToString());
			sensor.SetAttribute("Port", SensorPort);
			sensor.SetAttribute("Delay", ReadDelay.ToString());
			sensor.SetAttribute("MaxStepRate", MaxStepRate.ToString());
			sensor.SetAttribute("OnlySWR", ReadOnlySWR.ToString());
			sensor.SetAttribute("MinPower", MinPower.ToString());
			sensor.SetAttribute("Epsilon", SensorEpsilon.ToString());
			sensor.SetAttribute("AlarmCount", AlarmCount.ToString());
			root.AppendChild(sensor);

			// store radio details
			var radio = doc.CreateElement("Radio");
			radio.SetAttribute("Type", RadioType.ToString());
			radio.SetAttribute("Host", RadioHost);
			radio.SetAttribute("Port", RadioPort.ToString());
			radio.SetAttribute("Learning", RadioLearningEnabled.ToString());
			root.AppendChild(radio);

			return doc;
		}

		/// <summary>
		/// Deserialize the configuration from an XML structure.
		/// </summary>
		public bool Deserialize(XmlDocument doc) {
			foreach (var item in doc.DocumentElement.ChildNodes.OfType<XmlElement>()) {
				switch (item.LocalName) {
					case "Radio":
						RadioType = (RadioTypes)Enum.Parse(typeof(RadioTypes), item.GetAttribute("Type"));
						RadioHost = item.GetAttribute("Host");
						RadioPort = Int32.Parse(item.GetAttribute("Port"));
						if (item.HasAttribute("Learning")) {
							RadioLearningEnabled = Boolean.Parse(item.GetAttribute("Learning"));
						} else {
							RadioLearningEnabled = true; // default to 'on' when config doesn't specify.
						}
						break;
					case "Sensor":
						if (String.Equals(item.GetAttribute("Type"), "ArduinoV1", StringComparison.InvariantCultureIgnoreCase)) {
							// this case is for backwards compatibility with old "Arduino" based name
							SensorType = SensorTypes.AutoCapV1;
						} else {
							SensorType = (SensorTypes)Enum.Parse(typeof(SensorTypes), item.GetAttribute("Type"));
						}
						SensorPort = item.GetAttribute("Port");
						ReadDelay = Decimal.Parse(item.GetAttribute("Delay"));
						if (item.HasAttribute("MaxStepRate")) {
							MaxStepRate = UInt32.Parse(item.GetAttribute("MaxStepRate"));
						}
						MinPower = Decimal.Parse(item.GetAttribute("MinPower"));
						if (item.HasAttribute("Epsilon")) {
							SensorEpsilon = Decimal.Parse(item.GetAttribute("Epsilon"));
						}
						if (item.HasAttribute("OnlySWR")) {
							ReadOnlySWR = Boolean.Parse(item.GetAttribute("OnlySWR"));
						}
						if (item.HasAttribute("AlarmCount")) {
							AlarmCount = UInt16.Parse(item.GetAttribute("AlarmCount"));
						}
						break;
					case "Motor":
						if (String.Equals(item.GetAttribute("Type"), "ArduinoV1", StringComparison.InvariantCultureIgnoreCase)) {
							// this case is for backwards compatibility with old "Arduino" based name
							MotorType = MotorTypes.AutoCapV1;
						} else {
							MotorType = (MotorTypes)Enum.Parse(typeof(MotorTypes), item.GetAttribute("Type"));
						}
						if (item.HasAttribute("Port")) {
							MotorControlPort = item.GetAttribute("Port");
						} else if (item.HasAttribute("ControlPort")) {
							MotorControlPort = item.GetAttribute("ControlPort");
						}
						if (item.HasAttribute("OutputPort")) {
							MotorOutputPort = Int32.Parse(item.GetAttribute("OutputPort"));
						}
						MotorPolarityReversed = Boolean.Parse(item.GetAttribute("Reverse"));
						if (item.HasAttribute("ReverseRelay")) {
							MotorRelayRolesReversed = Boolean.Parse(item.GetAttribute("ReverseRelay"));
						}
						if (item.HasAttribute("BrakingEnabled")) {
							MotorBrakingEnabled = Boolean.Parse(item.GetAttribute("BrakingEnabled"));
						}
						break;
					case "Thresholds":
						var els = new LinkedList<XmlElement>(item.ChildNodes.OfType<XmlElement>());
						var nth = new Threshold[els.Count];
						var i = 0;
						foreach (var th in els) {
							nth[i++] = new Threshold {
								SWR = Decimal.Parse(th.GetAttribute("SWR")),
								Size = th.HasAttribute("Delay") ? Decimal.Parse(th.GetAttribute("Delay")) : Decimal.Parse(th.GetAttribute("Size")),
								Effort = th.HasAttribute("Effort") ? Decimal.Parse(th.GetAttribute("Effort")) : 1.0m,
								IsStep = th.HasAttribute("IsStep") ? Boolean.Parse(th.GetAttribute("IsStep")) : false
							};
						}
						Thresholds = nth;
						break;
				}
			}
			return true;
		}
	}
}
