﻿/*
 *
 *
 *    FlDigi.cs
 *
 *    Minimal FlDigi XMLRPC interface.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;

namespace KK5JY.RadioSupport {
	/// <summary>
	/// Basic FlDigi interface.
	/// </summary>
	public class FlDigi {
		#region Public Properties
		/// <summary>
		/// Fetch the frequency.
		/// </summary>
		public decimal Frequency {
			get {
				var s = GetFrequency();
				decimal d;
				if (Decimal.TryParse(s, out d)) {
					return Math.Truncate(d);
				}
				return 0m;
			}
		}
		#endregion

		#region Private Fields
		private string m_Host;
		private string m_Uri;
		private int m_Port;
		private byte[] m_FrequencyRequest;
		#endregion

		#region Constructors
		/// <summary>
		/// Create a new FlDigi remote object.
		/// </summary>
		/// <param name="host">The host name.</param>
		/// <param name="port">The port number.</param>
		public FlDigi(string host, int port = 7362) {
			m_Host = host;
			m_Port = port;
			m_Uri = String.Format("http://{0}:{1}/RPC2", m_Host, m_Port);
			using (var io = new MemoryStream()) {
				using (var w = new StreamWriter(io)) {
					w.WriteLine("<?xml version=\"1.0\" encoding=\"us-ascii\"?>");
					w.WriteLine("<methodCall><methodName>main.get_frequency</methodName><params></params></methodCall>");
					w.Flush();
				}
				m_FrequencyRequest = io.ToArray();
			}
		}
		#endregion

		#region Utilities
		/// <summary>
		/// Execute the XML-RPC needed to read the frequency from FLdigi.
		/// </summary>
		/// <returns></returns>
		private string GetFrequency() {
			try {
				using (var wc = new WebClient()) {
					var response = wc.UploadData(m_Uri, "POST", m_FrequencyRequest);
					var doc = new XmlDocument();
					using (var io = new MemoryStream(response)) {
						doc.Load(io);
						foreach (var element in doc.GetElementsByTagName("value").OfType<XmlElement>()) {
							return element.InnerText.Trim();
						}
					}
				}
			} catch {
				// nop, just fail
			}
			return "";
		}
		#endregion
	}
}
