﻿/*
 *
 *
 *    GenericSensor.cs
 *
 *    AutoCap - A generic sensor interface.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;

namespace KK5JY.AutoCap {
	/// <summary>
	/// A generic sensor.
	/// </summary>
	/// <remarks>
	/// The generic key-value pairs used for each update message:
	///     Power=n  - the forward power, measured in W
	///     SWR=n    - the current SWR; this is the only required field
	///     X=n      - the reactance, measured in ohms
	///     Z=n      - the impedance, measured in ohms
	///     Phi=n    - the phase angle, measured in degrees
	///     ID=s     - the serial number or other identifier for the sensor
	/// </remarks>
	public sealed class GenericSensor : GenericConnector, ISensor, IReportsVersion, IDisposable {
		#region Private Fields
		private SensorData m_Data;
		#endregion

		#region Public Events
		/// <summary>
		/// Raised when new sensor data is available.
		/// </summary>
		public event EventHandler<SensorDataEvent> SensorData;
		#endregion

		#region Public Properties
		/// <summary>
		/// Enable automatic polling.
		/// </summary>
		public bool AutoPoll { get; set; }

		/// <summary>
		/// Indicates that the sensor should only try to read SWR (not power, reactance, etc.)
		/// </summary>
		public bool ReadOnlySWR { get; set; }

		/// <summary>
		/// Return the version string.
		/// </summary>
		public string VersionString {
			get { return String.Format("Generic Sensor - {0}:{1}", EndPoint.Address, EndPoint.Port); }
		}
		#endregion

		#region Public Methods
		/// <summary>
		/// Create a new sensor.
		/// </summary>
		/// <param name="ep">The host to which to connect.</param>
		public GenericSensor(IPEndPoint ep) : base(ep) {
			m_Data = new SensorData();
		}

		/// <summary>
		/// Poll for new data.
		/// </summary>
		public void Poll() {
			if (SensorData != null) {
				SensorData(this, new SensorDataEvent { Data = m_Data });
			}
		}
		#endregion

		#region Event Handlers
		protected override void OnMalformedMessage(GenericMessageError e) {
			//
			//  TODO: increment a counter or something...
			//
		}

		protected override void OnMessageReceived(GenericMessageEvent e) {
			try {
				foreach (var kvp in e.Message) {
					if (String.Equals(kvp.Key, "Power", StringComparison.InvariantCultureIgnoreCase)) {
						m_Data.Power = Decimal.Parse(kvp.Value);
					} else if (String.Equals(kvp.Key, "SWR", StringComparison.InvariantCultureIgnoreCase)) {
						m_Data.SWR = Decimal.Parse(kvp.Value);
					} else if (String.Equals(kvp.Key, "X", StringComparison.InvariantCultureIgnoreCase)) {
						m_Data.X = Decimal.Parse(kvp.Value);
					} else if (String.Equals(kvp.Key, "Z", StringComparison.InvariantCultureIgnoreCase)) {
						m_Data.Z = Decimal.Parse(kvp.Value);
					} else if (String.Equals(kvp.Key, "Phi", StringComparison.InvariantCultureIgnoreCase)) {
						m_Data.Phi = Decimal.Parse(kvp.Value);
					} else if (String.Equals(kvp.Key, "ID", StringComparison.InvariantCultureIgnoreCase)) {
						m_Data.ID = kvp.Value.Trim();
					}
				}
				if (AutoPoll && SensorData != null) {
					SensorData(this, new SensorDataEvent { Data = m_Data });
				}
			} catch {
				//
				// TODO: increment a counter or something...
				//
			}
		}
		#endregion
	}
}
