﻿/*
 *
 *
 *    RS232Motor.cs
 *
 *    Simple RS-232 port motor interface, using handshake lines to toggle
 *    power and direction.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.IO.Ports;

namespace KK5JY.AutoCap {
	/// <summary>
	/// A motor controlled by RS-232 handshake lines.
	/// </summary>
	public class RS232Motor : IMotor, IReportsVersion, IDisposable {
		#region Public Types and Properties
		/// <summary>
		/// Wiring configuration options.
		/// </summary>
		public enum WireConfigOptions {
			/// <summary>
			/// DTR controls power, RTS controls polarity.
			/// </summary>
			DtrPowerRtsPolarity,

			/// <summary>
			/// RTS controls power, DTR controls polarity.
			/// </summary>
			RtsPowerDtrPolarity,
		}

		/// <summary>
		/// The wire configuration.
		/// </summary>
		public WireConfigOptions WireConfig { get; set; }

		/// <summary>
		/// Reverse the motor effort for movement commands.
		/// </summary>
		public bool MotorReversed { get; set; }

		/// <summary>
		/// Return a version string.
		/// </summary>
		public string VersionString { get; private set; }
		#endregion

		#region Private Fields
		/// <summary>
		/// The serial port object.
		/// </summary>
		private SerialPort m_Port;
		#endregion

		#region Public Methods
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="portName">The serial port name.</param>
		public RS232Motor(string portName) {
			WireConfig = WireConfigOptions.DtrPowerRtsPolarity;
			m_Port = new SerialPort(portName);
			m_Port.Open();

			// enter a safe state
			SetPower(false);
			SetDirection(false);

			// update the version string
			VersionString = String.Format("Generic RTS/DTR Interface ({0})", portName);
		}

		/// <summary>
		/// Close the serial port.
		/// </summary>
		public void Dispose() {
			try { m_Port.Close(); } catch { /* nop */ }
		}

		/// <summary>
		/// Move the motor.
		/// </summary>
		/// <param name="speed">The speed (in this case, just the direction).</param>
		public bool Move(decimal speed = 0) {
			if (MotorReversed) {
				speed *= -1m;
			}
			if (speed == 0m) {
				SetPower(false);
				return true;
			}
			if (speed < 1) {
				SetDirection(false);
			} else {
				SetDirection(true);
			}
			SetPower(true);
			return true;
		}
		#endregion

		#region Utilities
		private void SetDirection(bool polarity) {
			switch (WireConfig){
				case RS232Motor.WireConfigOptions.DtrPowerRtsPolarity:
					m_Port.RtsEnable = polarity;
					break;
				case RS232Motor.WireConfigOptions.RtsPowerDtrPolarity:
					m_Port.DtrEnable = polarity;
					break;
			}		
		}

		private void SetPower(bool onoff) {
			switch (WireConfig) {
				case RS232Motor.WireConfigOptions.DtrPowerRtsPolarity:
					m_Port.DtrEnable = onoff;
					break;
				case RS232Motor.WireConfigOptions.RtsPowerDtrPolarity:
					m_Port.RtsEnable = onoff;
					break;
			}
		}
		#endregion
	}
}
