﻿/*
 *
 *
 *    RS232MotorH.cs
 *
 *    Simple RS-232 port motor interface, using handshake lines to toggle
 *    the relays of a SPDT H-bridge.
 *
 *    License: GNU General Public License Version 3.0.
 *    
 *    Copyright (C) 2014 by Matthew K. Roberts, KK5JY. All rights reserved.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see: http://www.gnu.org/licenses/
 *    
 *
 */

using System;
using System.IO.Ports;

namespace KK5JY.AutoCap {
	/// <summary>
	/// A motor controlled by RS-232 handshake lines, connected to
    /// a relay-based H-bridge, or other simple up/down circuit.
	/// </summary>
	public class RS232MotorH : IMotor, IReportsVersion, IDisposable {
		#region Public Types and Properties
		/// <summary>
		/// Reverse the motor effort for movement commands.
		/// </summary>
		public bool MotorReversed { get; set; }

		/// <summary>
		/// Return a version string.
		/// </summary>
		public string VersionString { get; private set; }
		#endregion

		#region Private Fields
		/// <summary>
		/// The serial port object.
		/// </summary>
		private SerialPort m_Port;
		#endregion

		#region Public Methods
		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="portName">The serial port.</param>
		public RS232MotorH(string portName) {
			m_Port = new SerialPort(portName);
			m_Port.Open();

			// enter a safe state
            Move(0m);

			// update the version string
			VersionString = String.Format("Generic RTS/DTR H-Bridge Interface ({0})", portName);
		}

		/// <summary>
		/// Close the port.
		/// </summary>
		public void Dispose() {
			try { m_Port.Close(); } catch { /* nop */ }
		}

		/// <summary>
		/// Move the motor.
		/// </summary>
		/// <param name="speed">The speed (in this case, just the direction).</param>
		public bool Move(decimal speed = 0) {
			if (MotorReversed) {
				speed *= -1m;
			}
			if (speed == 0m) {
                SetRelay(0, false);
                SetRelay(1, false);
				return true;
			}
			if (speed < 1) {
                SetRelay(0, false);
                SetRelay(1, true);
			} else {
                SetRelay(0, true);
                SetRelay(1, false);
            }
			return true;
		}
		#endregion

		#region Utilities
        /// <summary>
        /// Update the relay state.
        /// </summary>
        /// <param name="number">The relay number, zero or one.</param>
        /// <param name="state">The new state on (true) or off (false).</param>
        private void SetRelay(int number, bool state)
        {
            switch (number)
            {
                case 0:
                    m_Port.DtrEnable = state;
                    break;
                case 1:
                    m_Port.RtsEnable = state;
                    break;
                default: break; // nop
            }
        }
		#endregion
	}
}
